@@ -3,7 +3,7 @@ use Carp;
 use strict;
 use Class::ISA;
 
-$CGI::Application::VERSION = '4.03';
+$CGI::Application::VERSION = '4.04';
 
 my %INSTALLED_CALLBACKS = (
 #	hook name          package                 sub
@@ -604,6 +604,9 @@ sub _send_headers {
 
 
 # Make all hash keys CAPITAL
+# although this method is internal, some other extensions
+# have come to rely on it, so any changes here should be
+# made with great care or avoided. 
 sub _cap_hash {
 	my $self = shift;
 	my $rhash = shift;
@@ -1324,13 +1327,27 @@ header_type() to "none".  This will completely hide headers.
 
 =item load_tmpl()
 
-    my $tmpl_obj = $webapp->load_tmpl('some.tmpl');
+    my $tmpl_obj = $webapp->load_tmpl;
+    my $tmpl_obj = $webapp->load_tmpl('some.html');
+    my $tmpl_obj = $webapp->load_tmpl( \$template_content );
+    my $tmpl_obj = $webapp->load_tmpl( FILEHANDLE );
 
-This method takes the name of a template file and returns an
-HTML::Template object.  ( For integration with other template systems
+This method takes the name of a template file, a reference to template data
+or a FILEHANDLE and returns an HTML::Template object. If the filename is undefined or missing, CGI::Application will default to trying to use the current run mode name, plus the extension ".html". 
+
+If you use the default template naming system, you should also use
+L<CGI::Application::Plugin::Forward>, which simply helps to keep the current
+name accurate when you pass control from one run mode to another.
+
+( For integration with other template systems
 and automated template names, see "Alternatives to load_tmpl() below. )
 
-The HTML::Template->new_file() constructor is used for create the object.
+When you pass in a filename, the HTML::Template->new_file() constructor
+is used for create the object.  When you pass in a reference to the template
+content, the HTML::Template->new_scalar_ref() constructor is used and
+when you pass in a filehandle, the HTML::Template->new_filehandle()
+constructor is used.
+
 Refer to L<HTML::Template> for specific usage of HTML::Template.
 
 If tmpl_path() has been specified, load_tmpl() will set the
@@ -1338,10 +1355,18 @@ HTML::Template C<path> option to the path(s) provided.  This further
 assists in encapsulating template usage.
 
 The load_tmpl() method will pass any extra parameters sent to it directly to
-HTML::Template->new_file().  This will allow the HTML::Template object to be
-further customized:
+HTML::Template->new_file() (or new_scalar_ref() or new_filehandle()).
+This will allow the HTML::Template object to be further customized:
 
-    my $tmpl_obj = $webapp->load_tmpl('some_other.tmpl',
+    my $tmpl_obj = $webapp->load_tmpl('some_other.html',
+         die_on_bad_params => 0,
+         cache => 1
+    );
+
+Note that if you want to pass extra arguments but use the default template
+name, you still need to provide a name of C<undef>:
+
+    my $tmpl_obj = $webapp->load_tmpl(undef',
          die_on_bad_params => 0,
          cache => 1
     );
@@ -1412,13 +1437,32 @@ sub load_tmpl {
 		push(@extra_params, path => [ @tmpl_paths ]) unless $found;
 	}
 
-    my %tmpl_params;
+    my %tmpl_params = ();
     my %ht_params = @extra_params;
+    %ht_params = () unless keys %ht_params;
+
+    # Define our extension if doesn't already exist;
+    $self->{__CURRENT_TMPL_EXTENSION} = '.html' unless defined $self->{__CURRENT_TMPL_EXTENSION};
+
+    # Define a default templat name based on the current run mode
+    unless (defined $tmpl_file) {
+        $tmpl_file = $self->get_current_runmode . $self->{__CURRENT_TMPL_EXTENSION};    
+    }
 
     $self->call_hook('load_tmpl', \%ht_params, \%tmpl_params, $tmpl_file);
 
     require HTML::Template;
-    my $t = HTML::Template->new_file($tmpl_file, %ht_params);
+    # let's check $tmpl_file and see what kind of parameter it is - we
+    # now support 3 options: scalar (filename), ref to scalar (the
+    # actual html/template content) and reference to FILEHANDLE
+    my $t = undef;
+    if ( ref $tmpl_file eq 'SCALAR' ) {
+        $t = HTML::Template->new_scalar_ref( $tmpl_file, %ht_params );
+    } elsif ( ref $tmpl_file eq 'GLOB' ) {
+        $t = HTML::Template->new_filehandle( $tmpl_file, %ht_params );
+    } else {
+        $t = HTML::Template->new_file($tmpl_file, %ht_params);
+    }
 
     if (keys %tmpl_params) {
         $t->param(%tmpl_params);
@@ -2135,6 +2179,7 @@ sub call_hook {
 			die "Error executing class callback in $hook stage: $@" if $@;
 		}
 	}
+
 }
 
 =pod
